module SubZero
  class Client

    attr_reader :sid, :config

    def initialize sid
      @sid, @config = sid.to_s.upcase, Configuration.default
      yield @config if block_given?
    end

    def socket
      Socket.new config
    end

    def call verb, payload = {}, options = {}
      sz_verb = verb.to_s.upcase.gsub('_', '/')

      request = Message.new(type: 'REQ', subtype: sid, verb: sz_verb,
                            payload: payload, options: options)

      response = socket.call(request)

      if block_given?
        yield response
      else
        fail Error.new(response) if response.error?
        response.payload
      end
    end

    def call! verb, payload = {}, options = {}, &block
      call verb, payload, options, &block
    end

    def [] verb
      call verb
    end

    def method_missing method, payload = {}, options = {}, &block
      if method.to_s.end_with? '!'
        call! method.to_s.delete('!'), payload, options, &block
      else
        call method, payload, options, &block
      end
    end

    class Error < ::StandardError

      attr_reader :result

      def initialize response
        message = response.payload['error'].try(:[], 'message')
        super message
        set_backtrace caller
        @result = response
      end

      def code
        result.payload['code']
      end

      def status
        ActiveSupport::StringInquirer.new self.code.to_s.downcase
      end

    end

  end
end
