require 'colorize'

module SubZero
  module Loggable

    COLORS = [:red, :green, :yellow, :blue, :magenta, :cyan,
      :light_red, :light_green, :light_yellow, :light_blue, :light_magenta, :light_cyan]

    SEVERITIES = [:info, :warn, :error, :debug]

    SEVERITY_COLORS = { info: :light_blue }

    SEVERITIES.each do |severity|
      define_method severity do |message, context = {}|
        log message, { severity: severity }.merge(context)
      end
    end

    def error exception, message
      super message, error: exception
    end

    def log message, context = {}
      sender_class = self.kind_of?(Class) ? self : self.class
      context.merge! sender_class: sender_class.to_s
      context[:severity] ||= :info

      puts log_colored_message(message, context)
    end

    def self.included(base)
      base.extend self
    end

    private

    def log_colored_message(message, context)
      msg = log_colored_severity(context[:severity])
      msg << " #{Time.now.strftime('%Y%m%d%H%M%S%L')}"

      if sender_class = context.delete(:sender_class)
        msg = log_colored_term(sender_class)
      end

      msg << " #{message}"
    end

    def log_colored_severity(severity)
      severity.to_s.colorize SEVERITY_COLORS[severity.to_sym]
    end

    def log_colored_term(term)
      index = Integer("0x#{Digest::MD5.hexdigest(term)[0, 2]}") % 12
      term.colorize COLORS[index]
    end

  end
end
