module SubZero
  class Message
    module Parser
      extend ActiveSupport::Concern

      def to_source
        frames = []

        if subtype.present?
          frames << "#{type}:#{subtype}"
        else
          frames << type
        end

        frames << rid

        if status.present?
          frames << "#{verb}:#{status}"
        else
          frames << verb
        end

        frames << payload.to_msgpack
        frames << options.to_msgpack

        frames
      end

      module ClassMethods

        def from_source source
          type = source.shift

          if type.index ':'
            type, subtype = type.split(':')
          end

          rid  = source.shift
          verb = source.shift

          if verb.index ':'
            verb, status = verb.split(':')
          end

          payload = MessagePack.unpack(source.shift).with_indifferent_access

          unless source.empty?
            routing_info = MessagePack.unpack(source.shift).with_indifferent_access
          end

          new type: type, rid: rid, verb: verb, payload: payload,
              subtype: subtype, status: status, routing_info: routing_info
        end

        def parse source, request = nil
          from_source(source).tap do |m|
            m.validate_response!(request) if request
          end
        end

      end

    end
  end
end
